#ifndef __EVAL_PARAMS_COMBINED_MODEL
#define __EVAL_PARAMS_COMBINED_MODEL

#include "definitions.h"
#include "charAndSeqModel.h"
#include "tree.h"
#include "sequenceContainer.h"






class evalParamsCombinedModel{
public:
	explicit evalParamsCombinedModel(const tree& tr, const sequenceContainer &charContainer, const sequenceContainer &seqContainer, charAndSeqModel* pCombinedModel, int mappingNum, paramType param, int heuristicGroups)
		:_tr(tr), _charContainer(charContainer),_seqContainer(seqContainer), _pCombinedModel(pCombinedModel), _param(param), _mappingNum(mappingNum), _heuristicGroups(heuristicGroups)
		{};
	MDOUBLE operator() (MDOUBLE x);

private:
	const tree& _tr;
	const sequenceContainer& _charContainer;
	const sequenceContainer& _seqContainer;
	charAndSeqModel* _pCombinedModel;
	paramType _param;
	int _mappingNum;
	int _heuristicGroups;
};



class evalParamsSeqModel{
public:
	explicit evalParamsSeqModel(const vector<tree> &inTrees, const Vdouble& weights, const sequenceContainer &seqContainer, charAndSeqModel* pCombinedModel, paramType param, const vector<tree> &charTrees, const sequenceContainer &charContainer, llCalc llType)
		:_inTrees(inTrees), _weights(weights), _seqContainer(seqContainer), _pCombinedModel(pCombinedModel), _param(param), _charTrees(charTrees), _charContainer(charContainer), _llType(llType)

		{};
	MDOUBLE operator() (MDOUBLE x);

private:
	const vector<tree> & _inTrees;
	const Vdouble& _weights;
	const sequenceContainer& _seqContainer;
	charAndSeqModel* _pCombinedModel;
	paramType _param;
	const vector<tree>& _charTrees;
	const sequenceContainer& _charContainer;
	llCalc _llType;
};


class evalParamsRelativeRate{
public:
	explicit evalParamsRelativeRate(const tree& inTree, const vector<treeMutationHistory>& mappings, const sequenceContainer &seqContainer, charAndSeqModel* pCombinedModel, int heuristicGroups)
		:_tr(inTree), _mappings(mappings), _seqContainer(seqContainer), _pCombinedModel(pCombinedModel), _heuristicGroups(heuristicGroups)
		{};
	MDOUBLE operator() (MDOUBLE x);

private:
	const tree& _tr;
    const vector<treeMutationHistory>& _mappings;
	const sequenceContainer& _seqContainer;
	charAndSeqModel* _pCombinedModel;
	int _heuristicGroups;
};


class evalParamsCombinedModelManyTrees{
public:
	explicit evalParamsCombinedModelManyTrees(const vector<tree>& inTrees, const sequenceContainer &charContainer, const sequenceContainer &seqContainer, charAndSeqModel* pCombinedModel, int mappingNum, paramType param, int heuristicGroups, llCalc llType)
		:_trees(inTrees), _charContainer(charContainer),_seqContainer(seqContainer), _pCombinedModel(pCombinedModel), _param(param), _mappingNum(mappingNum), _heuristicGroups(heuristicGroups), _llType(llType)
		{};
	MDOUBLE operator() (MDOUBLE x);

private:
	const vector<tree>& _trees;
	const sequenceContainer& _charContainer;
	const sequenceContainer& _seqContainer;
	charAndSeqModel* _pCombinedModel;
	paramType _param;
	int _mappingNum;
	int _heuristicGroups;
	llCalc _llType;
};


class evalParamsRelativeRateManyTrees{
public:
	explicit evalParamsRelativeRateManyTrees(const vector<tree>& inTrees, const vector<treeMutationHistory>& mappings, const sequenceContainer &seqContainer, charAndSeqModel* pCombinedModel, int heuristicGroups, const sequenceContainer &charContainer, llCalc llType)
		:_trees(inTrees), _mappings(mappings), _seqContainer(seqContainer), _pCombinedModel(pCombinedModel), _heuristicGroups(heuristicGroups), _charContainer(charContainer),_llType(llType)
		{};
	MDOUBLE operator() (MDOUBLE x);

private:
	const vector<tree>& _trees;
    const vector<treeMutationHistory>& _mappings;
	const sequenceContainer& _seqContainer;
	charAndSeqModel* _pCombinedModel;
	int _heuristicGroups;
	const sequenceContainer& _charContainer;
	llCalc _llType;
};

class evalParamsTreeLengthManyTrees{
public:
	explicit evalParamsTreeLengthManyTrees(const vector<tree>& inTrees, const vector<treeMutationHistory>& mappings, const sequenceContainer &seqContainer, charAndSeqModel* pCombinedModel, int heuristicGroups, const sequenceContainer &charContainer, llCalc llType)
		:_trees(inTrees), _mappings(mappings), _seqContainer(seqContainer), _pCombinedModel(pCombinedModel), _heuristicGroups(heuristicGroups), _charContainer(charContainer),_llType(llType)
		{};
	MDOUBLE operator() (MDOUBLE x);

private:
	const vector<tree>& _trees;
    const vector<treeMutationHistory>& _mappings;
	const sequenceContainer& _seqContainer;
	charAndSeqModel* _pCombinedModel;
	int _heuristicGroups;
	const sequenceContainer& _charContainer;
	llCalc _llType;
};

#endif
