#ifndef __K_MEAN_CLUSTERING
#define __K_MEAN_CLUSTERING
#include "definitions.h"



/***********************************************************************
This class implements the k-mean clustering algorithm: 
finds the k centers and the points that are closest to that center.
Input: (1) a set of points - each represented as an n-dimensional vector.
	   (2) k: Number of requested clusters
	   (3) iters: number of refinment iterations
The algorithm implements the K-MEANS++ algorithm: identical to the reguar k-mean but the initial points are chosen more wisely.
see http://en.wikipedia.org/wiki/K-means%2B%2B

In contrast to classic k-mean algorithms, here we the center of each cluster is one of the points belonging to that cluster, and not a novel point with the smallest distance to all other points
**********************************************************************/

class kMean {
public:
	kMean() {};   
    virtual ~kMean() {};
	//returns the centers and the number of points belonging to each center
	void run (const VVdouble& points, int k, int iters);

	const Vint& getCenters() const {return _centers;}
	const VVint& getClusterAssignments() const {return _clusterAssignments;}
	



private:
	void init(int pointsNum, int k);
	void calcDistances(const VVdouble& points);
	void chooseInitCenters(const VVdouble& points, int k);
	//in case k==1 - choose the point with the minimal distance to all other points
	void chooseOnePoint(const VVdouble& points);
	//a helper function to choose the initial centers
	int getNewCenterBasedOnDistances(const Vdouble& closestDist);
	void runKMeans(const VVdouble& points, int numIterations);
	//Given a collection of cluster centers, assigns each data point to the closest cluster center
	//returns true if one of the new assignments is different from the previous assignment
	bool  assignPoints2Centers(const VVdouble& points);
	void findNewCenters(const VVdouble& points);

private:
	VVdouble _distances;
	Vint _centers; //each center is represented as the index in the given points vector
	Vint _pointAssignments; //assignment of each point to one of the centers
	VVint _clusterAssignments;//for each cluster - list the indices of all points belonging to it
	Vint _id2cluster;//for each point - the index to its cluster assignment
};
#endif
