#ifndef ___SIMULATE_COMBINED_MODEL
#define ___SIMULATE_COMBINED_MODEL

#include "definitions.h"
#include "tree.h"
#include "sequenceContainer.h"
#include "charAndSeqModel.h"
#include "simulateTree.h"

#include <vector>
using namespace std;

/******************************************************************
This class simulates a combined character-and-sequence model along a tree and generates 2 sequence containers:
one for characters and one for sequences.
It first simulate he character model based on a jump simulation, so we now the exact point where a character transition occured. 
Then rescale the tree according to the relative rate of the character model.
Finally simulate sequences. 
*******************************************************************/

class simulateCombinedModel {
public:
	simulateCombinedModel(const tree& inTree, const charAndSeqModel* pCombinedModel, const alphabet* pCharAlph, const alphabet* pSeqAlph);
	virtual ~simulateCombinedModel();
	//simulate one character position and seqPosNum sequence positions
	//if overlap > 0 then simulate until the overlap between real and simulated characters is lower than the given threshold
	void simulate(int seqPosNum, const sequenceContainer* pOrigCharContainer=NULL, MDOUBLE overlap=-1); 
	sequenceContainer toSeqDataWithoutInternalNodes() const;
	sequenceContainer toCharDataWithoutInternalNodes() const;
	
private:
	void init();
	void simulateCharacterModel(treeMutationHistory& res, const sequenceContainer* pOrigCharContainer, MDOUBLE overlap);
	int getIdenticalSeqs(const sequenceContainer& sc1, const sequenceContainer& sc2, int pos);
	void simulateSon(tree::nodeP curNode, MDOUBLE disFromNode, int previousContent, int whichSon, treeMutationHistory& res);
	
	

private:
	const tree _tree;
	const charAndSeqModel* _pCombinedModel;
	const alphabet* _pCharAlph;
	const alphabet* _pSeqAlph;

	Vdouble _waitingTimeParams;//each entry is the lambda parameter of the exponential distribution modeling the waiting time for "getting out" of state i
	//_jumpProbs[i][j] is the probability of jumping from state i to state j (given that a change has ocured).
	VVdouble _jumpProbs; 

	Vint _nodesContent; // the actual character state at each node, retrieval according to node id
	sequenceContainer _charContainer;
	simulateTree* _pSeqSimulator; 
};

#endif
