#ifndef __TRAIT_RATE_MNG___H
#define __TRAIT_RATE_MNG___H

#include "definitions.h"
#include "tree.h"
#include "alphabet.h"
#include "charAndSeqModel.h"
#include "sequenceContainer.h"
#include "gainLossAlphabet.h"



class traitRateMng {

public:
	explicit traitRateMng();
	virtual ~traitRateMng();
    void run();
	void simulateSMIterations();
	void simulateRRFixedTaxa();
	void simulateNullModel(); //simulate a model with no rate difference 
	void simulateFalsePositive();//(1) simulate a trait that cause rate shift, (2) simulate sequences based on this trate, (3) simulate another trait 
	void calcLLDistribution(); //given a tree+MSAs+model param, calculate log-like large number of times to produce distribution of LL scores

	MDOUBLE getBestLL() const {return _bestLL;}

private:
	void init(); 
	void initAlphabet();
	void clear();
	void printOptionParameters();
	void validateInputs();
	void getTree();
	void getTrees(); //read multiple trees from file
	void getStartingData();
	void initStartingModel();
	//we have to make sure that the tree is in the correct unit and scale.
	//Thus: optimize the global rate inside distribution and the sequence stochastic process
	void optimizeTreeAndInitialSeqModel(); 
	MDOUBLE optimizeCombinedModel();
	void printResults();
	void printScaledTree(const string& scaledTreeStr);
	const vector<paramType> getOptimizedParams();

	MDOUBLE getInitialCharFreq();
	bool isCharModelReversible();
	
	void rootTree();
	
	//simulation related
	void getSimulatedTree(int iter);

private:
	charAndSeqModel* _pCombinedModel;
	sequenceContainer _seqContainer; 
	sequenceContainer _charContainer; 
	gainLossAlphabet _charAlph;
	alphabet* _pSeqAlph;
	tree _tree;
	vector<tree> _treeVec;
	//finiteIntAlphabet* _pAlph;
	MDOUBLE _bestLL;
	MDOUBLE _model0LL;
};
#endif
