#include "traitRateOptimizer.h"
#include "utils.h"
#include "numRec.h"
#include "likelihoodComputation.h"
#include "evalParamsCombinedModel.h"
#include "bestAlpha.h"
#include "bestHKYparam.h"
#include "bestGtrModelParams.h"
#include "traitRateOptions.h"
#include "treeUtil.h"
#include "talRandom.h"
#include <algorithm>
#include <ctime>
using namespace std;
/***********constructor***************/
traitRateOptimizer::traitRateOptimizer(const sequenceContainer &seqContainer, const sequenceContainer &charContainer)
: _seqContainer(seqContainer), _charContainer(charContainer)
{
}


//MDOUBLE traitRateOptimizer::optimizeCombinedModel(const vector<paramType>& params, const tree& inTree, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int mappingNum, int heuristicGroups)
//{
//	if (heuristicGroups == 1)
//		mappingNum *= 10;
//	MDOUBLE bestL = pCombinedModel->compLogLikelihood(inTree, _charContainer, _seqContainer, mappingNum, heuristicGroups);
//	LOG(3, <<"starting optimization combined model. LL = "<<bestL<<endl;); 
//	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
//
//	MDOUBLE prevIterL = VERYSMALL;
//	int iter;
//	for (iter = 0; iter < maxIter; ++iter) {
//        if (bestL < prevIterL + epsilonLLImprovement)
//		{
//			LOG(3, <<"#total number of optimizations:"<<iter<<endl;); 
//			return bestL; //likelihood converged
//		}
//		prevIterL = bestL;
//		LOG(3,<<endl<<"++++++++++iteration: "<<iter<<" begin"<<endl;);
//		//get a vector of stochastic mappings for the current character model params then optimize relative rate
//		stochasticMapping sm(inTree, *(pCombinedModel->getCharProcess()));
//		vector<treeMutationHistory> mappings;
//		sm.generateStochasticMapping(_charContainer, 0, mappingNum, mappings);
//		//optimize relative rate -> get vector of scaled trees given the optimize relative rate and character model
//		bestL = optimizeRelativeRate(bestL, params, mappings, inTree, pCombinedModel, epsilonLLImprovement, tolParamOptimization, heuristicGroups);
//		vector<tree> scaledTrees;
//		pCombinedModel->scaleTrees(inTree, mappings, pCombinedModel->getRelativeRate(), scaledTrees);
//		//optimize sequence model given the vector of scaled trees
//		bestL = optimizeSeqModel(bestL, params, scaledTrees, pCombinedModel, epsilonLLImprovement, 1, tolParamOptimization, heuristicGroups);
//		//optimize character model 
//		bestL = optimizeCharModel(bestL, params, inTree, pCombinedModel, epsilonLLImprovement, 1, tolParamOptimization, mappingNum, heuristicGroups);
//	}
//	LOG(3, <<"#total number of optimizations:"<<iter<<endl;); 
//	return bestL;
//}


MDOUBLE traitRateOptimizer::optimizeCombinedModel(const vector<paramType>& params, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int mappingNum, int heuristicGroups, llCalc llType)
{
	if (heuristicGroups == 1)
		mappingNum *= 10;
	vector<tree> randTrees = pickTrees(inTrees, mappingNum);
	MDOUBLE bestL = pCombinedModel->compLogLikelihood(randTrees, _charContainer, _seqContainer, mappingNum, heuristicGroups, randTrees, llType);
	LOG(3, <<"starting optimization combined model. LL = "<<bestL<<endl;); 
	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 

	MDOUBLE prevIterL = VERYSMALL;
	int iter;
	for (iter = 0; iter < maxIter; ++iter) {
        if (bestL < prevIterL + epsilonLLImprovement)
		{
			LOG(3, <<"#total number of optimizations:"<<iter<<endl;); 
			return bestL; //likelihood converged
		}
		prevIterL = bestL;
		LOG(3,<<endl<<"++++++++++iteration: "<<iter<<" begin"<<endl;);
		//get a vector of stochastic mappings for the current character model params then optimize relative rate
		vector<treeMutationHistory> mappings = stochasticMapping::getMappingForEachTree(randTrees, pCombinedModel->getCharProcess(), _charContainer, 0);
		//optimize relative rate -> get vector of scaled trees given the optimize relative rate and character model
		bestL = optimizeRelativeRate(bestL, params, mappings, randTrees, pCombinedModel, epsilonLLImprovement, tolParamOptimization, heuristicGroups, llType);
		bestL = optimizeTreeLength(bestL, params, mappings, randTrees, pCombinedModel, epsilonLLImprovement, tolParamOptimization, heuristicGroups, llType);
		cerr<<"after optimizeTreeLength"<<endl;
		vector<tree> scaledTrees;
		pCombinedModel->scaleTrees(randTrees, mappings, pCombinedModel->getRelativeRate(), scaledTrees);
		cerr<<"after scaleTrees"<<endl;
		//optimize sequence model given the vector of scaled trees
		//Vdouble weights(scaledTrees.size(), 1.0);
		//MDOUBLE bestL1 = pCombinedModel->compLogLikelihood(scaledTrees, _seqContainer, weights,  randTrees, _charContainer, llType);
		//cerr<<"after bestL1"<<endl;
		bestL = optimizeSeqModel(bestL, params, scaledTrees, pCombinedModel, epsilonLLImprovement, 1, tolParamOptimization, heuristicGroups, randTrees, llType);
		cerr<<"after optimizeSeqModel"<<endl;
		//MDOUBLE bestL2 = pCombinedModel->compLogLikelihood(scaledTrees, _seqContainer, weights,  randTrees, _charContainer, llType);
		//LOG(3, <<"bestL = "<<bestL<<"\t bestL1 = "<<bestL1<<" \t bestL2 = "<<bestL2<<endl;);
		//optimize character model 
		bestL = optimizeCharModel(bestL, params, randTrees, pCombinedModel, epsilonLLImprovement, 1, tolParamOptimization, mappingNum, heuristicGroups, llType);
	}
	LOG(3, <<"#total number of optimizations:"<<iter<<endl;); 
	return bestL;
}




//optimize the sequence process given a vector of trees with branch lengths scaled based on the stochastic mapping and the relativeRate
MDOUBLE traitRateOptimizer::optimizeSeqModel(MDOUBLE curLL, const vector<paramType>& params, const vector<tree> &inTrees1, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int heuristicGroups, const vector<tree> &charTrees, llCalc llType)
{
	vector<tree> scaledTrees;
	Vdouble weights;
	if (heuristicGroups == 1)
	{
		tree scaledTree = pCombinedModel->getAvgScaledTree(inTrees1);
		weights.push_back(1.0);
        scaledTrees.push_back(scaledTree);
	}
	else if (heuristicGroups > 1)
	{
		charAndSeqModel::getKMeanTrees(inTrees1, heuristicGroups, scaledTrees, weights);
	}
	else
	{
		weights.resize(inTrees1.size(), 1.0);
		scaledTrees = inTrees1;
	}
	vector<paramType> paramSet = getParamSet(params, 1);
	
    MDOUBLE bestL = curLL;
	if (bestL >= 0)
		bestL = pCombinedModel->compLogLikelihood(scaledTrees, _seqContainer, weights,  charTrees, _charContainer, llType);
	LOG(3, <<"starting optimization sequence model. LL = "<<bestL<<endl;); 
	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
	
	MDOUBLE prevIterL = VERYSMALL;
	int iter;
	for (iter = 0; iter < maxIter; ++iter) 
	{
	    if (bestL < prevIterL + epsilonLLImprovement)
			break; //likelihood converged
		prevIterL = bestL;
		LOG(10,<<"iteration: "<<iter<<" begin"<<endl;);
		for(int p = 0; p < paramSet.size(); ++p) {
			MDOUBLE paramFound;
			string paramName = pCombinedModel->getParamName(paramSet[p]);
			LOG(3,<<"optmizing "<<paramName<<endl;);
			MDOUBLE newL;
            MDOUBLE oldVal = pCombinedModel->getParam(paramSet[p]);
			MDOUBLE lowerBound = pCombinedModel->getParamLowerBound(paramSet[p]);
			MDOUBLE upperBound = pCombinedModel->getParamUpperBound(paramSet[p]);
			newL = -brent(lowerBound, oldVal, upperBound, evalParamsSeqModel(scaledTrees, weights, _seqContainer, pCombinedModel, paramSet[p], charTrees, _charContainer, llType),tolParamOptimization, &paramFound); 	
            if (newL >= bestL) 
			{
	            bestL = newL;
		        pCombinedModel->setParam(paramSet[p], paramFound);
			}
			else
			{//likelihood went down!
				LOG(3,<<"like went down when optimizing: "<<paramName<<endl;);
				LOG(3,<<"param Found = "<<paramFound<<"   LL ="<<newL<<"..."<<endl;);
				LOG(3,<<"param  old = "<<oldVal<<"   LL="<<bestL<<"..."<<endl;);
				LOGDO(3, pCombinedModel->printModelParams(myLog::LogFile())); 
				//errorMsg::reportError("lL went down!!!");
				pCombinedModel->setParam(paramSet[p], oldVal);
			}
	        LOG(3,<<" LL= "<<bestL<<" new = "<<paramFound<<" old="<<oldVal<<endl;);
		}
	}
	LOG(5, <<"ll after optimization = "<<bestL<<endl;);
	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
	return bestL;
}


//MDOUBLE traitRateOptimizer::optimizeCharModel(MDOUBLE curLL, const vector<paramType>& params, const tree& inTree, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int mappingNum, int heuristicGroups)
//{
//	//vector<paramType>& params getParams(
//	vector<paramType> paramSet = getParamSet(params, 2);
//	if (paramSet.empty())
//		return -1;
//	cerr<<"+++++++++++++++++++++++++++++++++++++++++"<<endl;
//	LOG(3, <<"starting optimization character model. LL = "<<curLL<<endl;); 
//	//MDOUBLE bestL = pCombinedModel->compLogLikelihood(inTree, _charContainer, _seqContainer, mappingNum, heuristicGroups);
//	MDOUBLE bestL = curLL;
//	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
//	
//	MDOUBLE prevIterL = VERYSMALL;
//	int iter;
//	for (iter = 0; iter < maxIter; ++iter) 
//	{
//	    if (bestL < prevIterL + epsilonLLImprovement)
//			break; //likelihood converged
//		prevIterL = bestL;
//		LOG(10,<<"iteration: "<<iter<<" begin"<<endl;);
//		for(int p = 0; p < paramSet.size(); ++p) {
//			MDOUBLE paramFound;
//			string paramName = pCombinedModel->getParamName(paramSet[p]);
//			LOG(3,<<"optmizing "<<paramName<<endl;);
//			MDOUBLE newL;
//            MDOUBLE oldVal = pCombinedModel->getParam(paramSet[p]);
//            MDOUBLE lowerBound = pCombinedModel->getParamLowerBound(paramSet[p]);
//			MDOUBLE upperBound = pCombinedModel->getParamUpperBound(paramSet[p]);
//			newL = -brent(lowerBound, oldVal, upperBound, evalParamsCombinedModel(inTree, _charContainer, _seqContainer, pCombinedModel, mappingNum, paramSet[p], heuristicGroups),tolParamOptimization, &paramFound); 	
//            if (newL >= bestL) 
//			{
//	            bestL = newL;
//		        pCombinedModel->setParam(paramSet[p], paramFound);
//			}
//			else
//			{//likelihood went down!
//				pCombinedModel->setParam(paramSet[p], oldVal);
//				LOGnOUT(3,<<"like went down when optimizing: "<<paramName<<endl;);
//				LOGnOUT(3,<<"param Found = "<<paramFound<<"   LL ="<<newL<<"..."<<endl;);
//				LOGnOUT(3,<<"param  old = "<<oldVal<<"   LL="<<bestL<<"..."<<endl<<endl;);
//				LOGDO(3, pCombinedModel->printModelParams(myLog::LogFile())); 
//				if (bestL-newL > 10)
//				{
//					//errorMsg::reportError("LL went down by more than 10 LL point!!!");
//					cerr<<endl<<"LL went down by more than 10 LL point!!!"<<endl;
//				}
//			}
//	        LOG(3,<<" LL= "<<bestL<<" new = "<<paramFound<<" old="<<oldVal<<endl;);
//		}
//	}
//	LOG(5, <<"ll after optimizating character model = "<<bestL<<endl;);
//	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
//	return bestL;
//}


MDOUBLE traitRateOptimizer::optimizeCharModel(MDOUBLE curLL, const vector<paramType>& params, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int mappingNum, int heuristicGroups, llCalc llType)
{
	if ((llType == SEQ) && (pCombinedModel->getCombinedModelType() == M0))
		return curLL;
	//vector<paramType>& params getParams(
	vector<paramType> paramSet = getParamSet(params, 2);
	if (paramSet.empty())
		return -1;
	cerr<<"+++++++++++++++++++++++++++++++++++++++++"<<endl;
	LOG(3, <<"starting optimization character model. LL = "<<curLL<<endl;); 
	//MDOUBLE bestL = pCombinedModel->compLogLikelihood(inTree, _charContainer, _seqContainer, mappingNum, heuristicGroups);
	MDOUBLE bestL = curLL;
	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
	
	MDOUBLE prevIterL = VERYSMALL;
	int iter;
	for (iter = 0; iter < maxIter; ++iter) 
	{
	    if (bestL < prevIterL + epsilonLLImprovement)
			break; //likelihood converged
		prevIterL = bestL;
		LOG(10,<<"iteration: "<<iter<<" begin"<<endl;);
		for(int p = 0; p < paramSet.size(); ++p) {
			MDOUBLE paramFound;
			string paramName = pCombinedModel->getParamName(paramSet[p]);
			LOG(3,<<"optmizing "<<paramName<<endl;);
			MDOUBLE newL;
            MDOUBLE oldVal = pCombinedModel->getParam(paramSet[p]);
            MDOUBLE lowerBound = pCombinedModel->getParamLowerBound(paramSet[p]);
			MDOUBLE upperBound = pCombinedModel->getParamUpperBound(paramSet[p]);
			newL = -brent(lowerBound, oldVal, upperBound, evalParamsCombinedModelManyTrees(inTrees, _charContainer, _seqContainer, pCombinedModel, mappingNum, paramSet[p], heuristicGroups, llType),tolParamOptimization, &paramFound); 	
            if (newL >= bestL) 
			{
	            bestL = newL;
		        pCombinedModel->setParam(paramSet[p], paramFound);
			}
			else
			{//likelihood went down!
				pCombinedModel->setParam(paramSet[p], oldVal);
				LOGnOUT(3,<<"like went down when optimizing: "<<paramName<<endl;);
				LOGnOUT(3,<<"param Found = "<<paramFound<<"   LL ="<<newL<<"..."<<endl;);
				LOGnOUT(3,<<"param  old = "<<oldVal<<"   LL="<<bestL<<"..."<<endl<<endl;);
				LOGDO(3, pCombinedModel->printModelParams(myLog::LogFile())); 
				if (bestL-newL > 10)
				{
					//errorMsg::reportError("LL went down by more than 10 LL point!!!");
					LOG(3, <<"LL went down by more than 10 LL point!!!");
				}
			}
	        LOG(3,<<" LL= "<<bestL<<" new = "<<paramFound<<" old="<<oldVal<<endl;);
		}
	}
	LOG(5, <<"ll after optimizating character model = "<<bestL<<endl;);
	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
	return bestL;
}


//optimize the relative rate paramete given pre-generated stochastic mappings
//MDOUBLE traitRateOptimizer::optimizeRelativeRate(MDOUBLE curLL, const vector<paramType>& params, const vector<treeMutationHistory>& mappings, const tree& inTree, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, MDOUBLE tolParamOptimization, int heuristicGroups)
//{
//	MDOUBLE res = VERYSMALL;
//	vector<paramType> paramSet = getParamSet(params, 3);
//	if (paramSet.size() == 0)
//		return -1;
//	if (paramSet.size() != 1)
//		errorMsg::reportError("unknown parameter in traitRateOptimizer::optimizeRelativeRate");
//	LOG(3, <<"starting optimization relative rate. LL = "<<curLL<<endl;); 
//	//curLL = pCombinedModel->compLogLikelihood(inTree, mappings, _seqContainer, heuristicGroups);
//	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
//
//	MDOUBLE oldParam = pCombinedModel->getRelativeRate();
//	MDOUBLE paramFound = 0;
//	MDOUBLE lowerBound = pCombinedModel->getParamLowerBound(RELATIVE_RATE);
//	MDOUBLE upperBound = pCombinedModel->getParamUpperBound(RELATIVE_RATE);
//
//	MDOUBLE newL = -brent(lowerBound, oldParam, upperBound, evalParamsRelativeRate(inTree, mappings, _seqContainer, pCombinedModel, heuristicGroups), tolParamOptimization, &paramFound); 
//	if (newL >= curLL) 
//	{
//		pCombinedModel->setRelativeRate(paramFound);
//		res = newL;
//	} 
//	else //likelihood went down!
//	{
//		res = curLL;
//		pCombinedModel->setRelativeRate(oldParam);
//		LOG(3,<<"like went down when optmizing relative rate"<<endl;);
//		LOG(3,<<"param Found... "<<paramFound<<"...LL ="<<newL<<"..."<<endl;);
//		LOG(3,<<"old param... "<<oldParam<<"...LL="<<curLL<<"..."<<endl;);
//	}
//	LOG(5, <<"ll after optimizing relative rate = "<<res<<endl;);
//	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
//	return res;
//}


//optimize the relative rate paramete given pre-generated stochastic mappings
MDOUBLE traitRateOptimizer::optimizeRelativeRate(MDOUBLE curLL, const vector<paramType>& params, const vector<treeMutationHistory>& mappings, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, MDOUBLE tolParamOptimization, int heuristicGroups, llCalc llType)
{
	if (pCombinedModel->getCombinedModelType() == M0)
		return curLL;
	MDOUBLE res = VERYSMALL;
	vector<paramType> paramSet = getParamSet(params, 3);
	if (paramSet.size() == 0)
		return -1;
	if (paramSet.size() != 1)
		errorMsg::reportError("unknown parameter in traitRateOptimizer::optimizeRelativeRate");
	LOG(3, <<"starting optimization relative rate. LL = "<<curLL<<endl;); 
	//curLL = pCombinedModel->compLogLikelihood(inTree, mappings, _seqContainer, heuristicGroups);
	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 

	MDOUBLE oldParam = pCombinedModel->getRelativeRate();
	MDOUBLE paramFound = 0;
	MDOUBLE lowerBound = pCombinedModel->getParamLowerBound(RELATIVE_RATE);
	MDOUBLE upperBound = pCombinedModel->getParamUpperBound(RELATIVE_RATE);

	MDOUBLE newL = -brent(lowerBound, oldParam, upperBound, evalParamsRelativeRateManyTrees(inTrees, mappings, _seqContainer, pCombinedModel, heuristicGroups, _charContainer, llType), tolParamOptimization, &paramFound); 
	if (newL >= curLL) 
	{
		pCombinedModel->setRelativeRate(paramFound);
		res = newL;
	} 
	else //likelihood went down!
	{
		res = curLL;
		pCombinedModel->setRelativeRate(oldParam);
		LOG(3,<<"like went down when optmizing relative rate"<<endl;);
		LOG(3,<<"param Found... "<<paramFound<<"...LL ="<<newL<<"..."<<endl;);
		LOG(3,<<"old param... "<<oldParam<<"...LL="<<curLL<<"..."<<endl;);
	}
	LOG(5, <<"ll after optimizing relative rate = "<<res<<endl;);
	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
	return res;
}


//optimize the total tree length parameter given pre-generated stochastic mappings
MDOUBLE traitRateOptimizer::optimizeTreeLength(MDOUBLE curLL, const vector<paramType>& params, const vector<treeMutationHistory>& mappings, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, MDOUBLE tolParamOptimization, int heuristicGroups, llCalc llType)
{
	MDOUBLE res = VERYSMALL;
	vector<paramType> paramSet = getParamSet(params, 4);
	if (paramSet.size() == 0)
		return -1;
	if (paramSet.size() != 1)
		errorMsg::reportError("unknown parameter in traitRateOptimizer::optimizeTreeLength");
	LOG(3, <<"starting optimization tree length. LL = "<<curLL<<endl;); 
	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 

	MDOUBLE oldParam = pCombinedModel->getTreeLength();
	MDOUBLE paramFound = 0;
	MDOUBLE lowerBound = pCombinedModel->getParamLowerBound(TREE_LENGTH);
	MDOUBLE upperBound = pCombinedModel->getParamUpperBound(TREE_LENGTH);

	MDOUBLE newL = -brent(lowerBound, oldParam, upperBound, evalParamsTreeLengthManyTrees(inTrees, mappings, _seqContainer, pCombinedModel, heuristicGroups, _charContainer, llType), tolParamOptimization, &paramFound); 
	if (newL >= curLL) 
	{
		pCombinedModel->setTreeLength(paramFound);
		res = newL;
	} 
	else //likelihood went down!
	{
		res = curLL;
		pCombinedModel->setTreeLength(oldParam);
		LOG(3,<<"like went down when optmizing tree length"<<endl;);
		LOG(3,<<"param Found... "<<paramFound<<"...LL ="<<newL<<"..."<<endl;);
		LOG(3,<<"old param... "<<oldParam<<"...LL="<<curLL<<"..."<<endl;);
	}
	LOG(5, <<"ll after optimizing tree length = "<<res<<endl;);
	LOGDO(5, pCombinedModel->printModelParams(myLog::LogFile())); 
	return res;
}



//Given the type (1 = sequence, 2 = character, 3 = relativeRate) get the appropriate subset of parameters from the input params
vector<paramType> traitRateOptimizer::getParamSet(const vector<paramType>& params, int type)
{
	vector<paramType> res;
	for (int p = 0; p < params.size(); ++p)
	{
		switch (type)
		{
		case 1:
			switch (params[p])
			{
			case GLOBAL_RATE:
			case SEQ_MODEL_ASRV:
			case HKY_MODEL_PARAM:
			case GTR_MODEL_PARAM1:
			case GTR_MODEL_PARAM2:
			case GTR_MODEL_PARAM3:
			case GTR_MODEL_PARAM4:
			case GTR_MODEL_PARAM5:
			case GTR_MODEL_PARAM6:
				res.push_back(params[p]);
				break;
			}
			break;
		case 2:
			switch (params[p])
			{
			case CHAR_MODEL_PARAM1:
			case CHAR_MODEL_PARAM2:
				res.push_back(params[p]);
				break;
			}
			break;
		case 3:
			if (params[p] == RELATIVE_RATE)
				res.push_back(params[p]);
			break;
		case 4:
			if (params[p] == TREE_LENGTH)
				res.push_back(params[p]);
			break;
		}
	}
	return res;
}



//MDOUBLE traitRateOptimizer::optimizeModelManyStart(const vector<paramType>& params, const tree& inTree, charAndSeqModel* pCombinedModel, const Vint& pointsNum, const Vint& iterNum, const Vdouble& tols, const Vint& heurGroups, int mappingNum)
//{
//	//make sure that the number of points in each cycle is not bigger than the previous cycle.
//	int i;
//	for (i = 0; i < pointsNum.size()-1; ++i) {
//		if (pointsNum[i] < pointsNum[i+1])
//			errorMsg::reportError("input error in chrNumOptimizer::optimizeModelManyStart()");
//	}
//	//create starting models
//	vector<charAndSeqModel*> modelVec;
//	for (int i = 0; i < pointsNum[0]; ++i)
//	{
//		//the first model is identical to the current one
//		if (i == 0)
//			modelVec.push_back(pCombinedModel->clone());
//		else
//			modelVec.push_back(getRandomModel(pCombinedModel)); 
//	}
//	
//	int numOfOptCycles = pointsNum.size();
//	Vdouble likelihoodVec;
//	for (int n = 0; n < numOfOptCycles; ++n)
//	{
//		time_t t1;
//		time(&t1);
//		LOG(5, <<"=====Cycle======= " <<n<<endl;);
//		if (n != 0)
//		{
//			//sort results and continue optimization only with the best (pointsNum[i]) points
//			vector<charAndSeqModel*> tmpModelVec(0); //store temporarily the best pointsNum[i] sp* 
//			Vdouble sortedL = likelihoodVec;
//			sort(sortedL.begin(),sortedL.end());
//			LOG(5, <<"Best Likelihood after cycle: "<<n-1<<" = "<<sortedL[sortedL.size()-1]<<endl);
//			MDOUBLE threshold = sortedL[sortedL.size()- pointsNum[n]];
//			for (int j = 0; j < likelihoodVec.size(); ++j)
//			{
//				if (likelihoodVec[j] >= threshold) 
//					tmpModelVec.push_back(modelVec[j]);
//				else
//					delete modelVec[j];
//			}
//			modelVec.clear();
//			modelVec = tmpModelVec;
//		} 
//
//		likelihoodVec.clear();
//		likelihoodVec.resize(pointsNum[n]); 
//		for (int c = 0; c < pointsNum[n]; ++c)
//		{
//			LOG(3, <<"=====optimizing point======= " <<c<<endl;);
//			MDOUBLE ll = optimizeCombinedModel(params, inTree, modelVec[c], tols[n], iterNum[n], tols[n], mappingNum, heurGroups[n]);
//			//MDOUBLE ll = optimizeCombinedModel(params, inTree, modelVec[c], tols[n], iterNum[n], tols[n], mappingNum, heurGroups[n]);
//            LOG(3, <<"point: "<<c<<"  likelihood = "<<ll<<endl<<endl;);
//			likelihoodVec[c] = ll;
//		}
//		time_t t2;
//		time(&t2);
//		LOGnOUT(5,<<endl<<"#RUNNING TIME OF CYCLE "<<n<<"="<<static_cast<int>(t2-t1)<<endl);
//	}
//	//finish optimization - get best model
//	Vdouble sortedL = likelihoodVec;
//	sort(sortedL.begin(),sortedL.end());
//	MDOUBLE bestL = sortedL[likelihoodVec.size() - 1];
//	LOG(3, <<endl<<"FINAL LIKELIHOODS++++++++++++++"<<endl;);
//	for (int i = 0; i < likelihoodVec.size(); ++i)
//	{
//		if (bestL == likelihoodVec[i]) 
//		{
//			//this operator= delete the current pointers of the model and clone new ones
//			*pCombinedModel = *modelVec[i];
//		}
//		delete modelVec[i];
//		modelVec[i] = NULL;
//		LOG(5, <<"point "<<i<<" likelihood = "<<likelihoodVec[i]<<endl;);
//		
//	}	
//	modelVec.clear();
//	return bestL;
//}
//

MDOUBLE traitRateOptimizer::optimizeModelManyStart(const vector<paramType>& params, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, const Vint& pointsNum, const Vint& iterNum, const Vdouble& tols, const Vint& heurGroups, int mappingNum, llCalc llType)
{
	//make sure that the number of points in each cycle is not bigger than the previous cycle.
	int i;
	for (i = 0; i < pointsNum.size()-1; ++i) {
		if (pointsNum[i] < pointsNum[i+1])
			errorMsg::reportError("input error in chrNumOptimizer::optimizeModelManyStart()");
	}
	//create starting models
	vector<charAndSeqModel*> modelVec;
	for (int i = 0; i < pointsNum[0]; ++i)
	{
		//the first model is identical to the current one
		if (i == 0)
			modelVec.push_back(pCombinedModel->clone());
		else
			modelVec.push_back(getRandomModel(pCombinedModel)); 
	}
	
	int numOfOptCycles = pointsNum.size();
	Vdouble likelihoodVec;
	for (int n = 0; n < numOfOptCycles; ++n)
	{
		time_t t1;
		time(&t1);
		LOG(5, <<"=====Cycle======= " <<n<<endl;);
		if (n != 0)
		{
			//sort results and continue optimization only with the best (pointsNum[i]) points
			vector<charAndSeqModel*> tmpModelVec(0); //store temporarily the best pointsNum[i] sp* 
			Vdouble sortedL = likelihoodVec;
			sort(sortedL.begin(),sortedL.end());
			LOG(5, <<"Best Likelihood after cycle: "<<n-1<<" = "<<sortedL[sortedL.size()-1]<<endl);
			MDOUBLE threshold = sortedL[sortedL.size()- pointsNum[n]];
			for (int j = 0; j < likelihoodVec.size(); ++j)
			{
				if (likelihoodVec[j] >= threshold) 
					tmpModelVec.push_back(modelVec[j]);
				else
					delete modelVec[j];
			}
			modelVec.clear();
			modelVec = tmpModelVec;
		} 

		likelihoodVec.clear();
		likelihoodVec.resize(pointsNum[n]); 
		for (int c = 0; c < pointsNum[n]; ++c)
		{
			LOG(3, <<"=====optimizing point======= " <<c<<endl;);
			MDOUBLE ll = optimizeCombinedModel(params, inTrees, modelVec[c], tols[n], iterNum[n], tols[n], mappingNum, heurGroups[n], llType);
			//MDOUBLE ll = optimizeCombinedModel(params, inTree, modelVec[c], tols[n], iterNum[n], tols[n], mappingNum, heurGroups[n]);
            LOG(3, <<"point: "<<c<<"  likelihood = "<<ll<<endl<<endl;);
			likelihoodVec[c] = ll;
		}
		time_t t2;
		time(&t2);
		LOGnOUT(5,<<endl<<"#RUNNING TIME OF CYCLE "<<n<<"="<<static_cast<int>(t2-t1)<<endl);
	}
	//finish optimization - get best model
	Vdouble sortedL = likelihoodVec;
	sort(sortedL.begin(),sortedL.end());
	MDOUBLE bestL = sortedL[likelihoodVec.size() - 1];
	LOG(3, <<endl<<"FINAL LIKELIHOODS++++++++++++++"<<endl;);
	for (int i = 0; i < likelihoodVec.size(); ++i)
	{
		if (bestL == likelihoodVec[i]) 
		{
			//this operator= delete the current pointers of the model and clone new ones
			*pCombinedModel = *modelVec[i];
		}
		delete modelVec[i];
		modelVec[i] = NULL;
		LOG(5, <<"point "<<i<<" likelihood = "<<likelihoodVec[i]<<endl;);
		
	}	
	modelVec.clear();
	return bestL;
}



charAndSeqModel* traitRateOptimizer::getRandomModel(const charAndSeqModel* pBaseModel) const
{
	charAndSeqModel* pRes = pBaseModel->clone();
	pRes->setRandomRateParams();
	return pRes;

}

//get random treeNum trees from inTrees
vector<tree> traitRateOptimizer::pickTrees(const vector<tree>& inTrees, int treeNum)
{
	if (inTrees.size() == treeNum)
		return inTrees;
	vector<tree> res;
	for (int t = 0; t < treeNum; ++t)
	{
		if (inTrees.size() == 1) {
			res.push_back(inTrees[0]);
		}
		else
		{
            int randT = talRandom::giveIntRandomNumberBetweenZeroAndEntry(inTrees.size()-1);
            res.push_back(inTrees[randT]);
		}
	}
	return res;
}
