#ifndef __TRAIT_RATE_OPTIMIER_H
#define __TRAIT_RATE_OPTIMIER_H

#include "definitions.h"
#include "charAndSeqModel.h"
#include "evalParamsCombinedModel.h"
#include "tree.h"
#include "sequenceContainer.h"
#include "stochasticProcess.h"

/***********************************************************************
 **********************************************************************/
class traitRateOptimizer {
public:
	explicit traitRateOptimizer (const sequenceContainer &seqContainer, const sequenceContainer &charContainer);
	virtual ~traitRateOptimizer() {};
	
	//MDOUBLE optimizeCombinedModel(const vector<paramType>& params, const tree& inTree, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int mappingNum, int heuristicGroups);
	MDOUBLE optimizeCombinedModel(const vector<paramType>& params, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int mappingNum, int heuristicGroups, llCalc llType);
	//MDOUBLE optimizeModelManyStart(const vector<paramType>& params, const tree& inTree, charAndSeqModel* pCombinedModel, const Vint& pointsNum, const Vint& iterNum, const Vdouble& tols, const Vint& heuristicGroups, int mappingNum);
	MDOUBLE optimizeModelManyStart(const vector<paramType>& params, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, const Vint& pointsNum, const Vint& iterNum, const Vdouble& tols, const Vint& heuristicGroups, int mappingNum, llCalc llType);
	//optimize the sequence process given a vector of trees with branch lengths scaled based on the stochastic mapping and the relativeRate
	MDOUBLE optimizeSeqModel(MDOUBLE curLL, const vector<paramType>& params, const vector<tree> &inTrees, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int heuristicGroups, const vector<tree> &charTrees, llCalc llType);
	//get random treeNum trees from inTrees
	static vector<tree> pickTrees(const vector<tree>& inTrees, int treeNum);

private:
	//optimize the parameters of the character models. 
	MDOUBLE optimizeCharModel(MDOUBLE curLL, const vector<paramType>& params, const tree& inTree, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int mappingNum, int heuristicGroups, llCalc llType);
	MDOUBLE optimizeCharModel(MDOUBLE curLL, const vector<paramType>& params, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, int maxIter, MDOUBLE tolParamOptimization, int mappingNum, int heuristicGroups, llCalc llType);
	//optimize the relative rate paramete given pre-generated stochastic mappings
	//MDOUBLE optimizeRelativeRate(MDOUBLE curLL, const vector<paramType>& params, const vector<treeMutationHistory>& mappings, const tree& inTree, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, MDOUBLE tolParamOptimization, int heuristicGroups);
	MDOUBLE optimizeRelativeRate(MDOUBLE curLL, const vector<paramType>& params, const vector<treeMutationHistory>& mappings, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, MDOUBLE tolParamOptimization, int heuristicGroups, llCalc llType);
	MDOUBLE optimizeTreeLength(MDOUBLE curLL, const vector<paramType>& params, const vector<treeMutationHistory>& mappings, const vector<tree>& inTrees, charAndSeqModel* pCombinedModel, MDOUBLE epsilonLLImprovement, MDOUBLE tolParamOptimization, int heuristicGroups, llCalc llType);

	//Given the type (1 = sequence, 2 = character, 3 = relativeRate) get the appropriate subset of parameters from the input params
	vector<paramType> getParamSet(const vector<paramType>& params, int type);
	charAndSeqModel* getRandomModel(const charAndSeqModel* pCombinedModel) const;
private:
	sequenceContainer _seqContainer;
	sequenceContainer _charContainer;
};
#endif
