#include "traitRateOptions.h"
#include "errorMsg.h"
#include "someUtil.h"
#include "Parameters.h"
#include <iostream>
using namespace std;

string traitRateOptions::_mainType;
string traitRateOptions::_treeFile;
string traitRateOptions::_treesFile;
string traitRateOptions::_characterFile;
string traitRateOptions::_seqFile;
string traitRateOptions::_logFile;
int traitRateOptions::_logValue;
string traitRateOptions::_outDir;
string traitRateOptions::_outFile;
string traitRateOptions::_scaledTreeFile;
twoStateModel::rootFreqType traitRateOptions::_rootFreqType;
combinedModelType traitRateOptions::_modelType;
string traitRateOptions::_rootAt;
MDOUBLE traitRateOptions::_relRate;
MDOUBLE traitRateOptions::_charModelParam1;
MDOUBLE traitRateOptions::_charModelParam2;
MDOUBLE traitRateOptions::_globalRate;
MDOUBLE traitRateOptions::_seqModelParam1; 
MDOUBLE traitRateOptions::_seqModelParam2; 
MDOUBLE traitRateOptions::_seqModelParam3; 
MDOUBLE traitRateOptions::_seqModelParam4; 
MDOUBLE traitRateOptions::_seqModelParam5; 
MDOUBLE traitRateOptions::_seqModelParam6; 
int traitRateOptions::_gammaCategories; 
MDOUBLE traitRateOptions::_gammaParam; 
MDOUBLE traitRateOptions::_treeLength;
string traitRateOptions::_referenceTree;
int traitRateOptions::_stochasicMappingIterations;
bool traitRateOptions::_bScaleTree; 
bool traitRateOptions::_bOptRelRate; 
bool traitRateOptions::_bOptSeqModel; 
bool traitRateOptions::_bOptCharModel; 
int traitRateOptions::_maxOptimizationIterations; 
MDOUBLE traitRateOptions::_epsilonLLimprovement; 
MDOUBLE traitRateOptions::_tolParamOptimization; 
Vint traitRateOptions::_optimizePointsNum; 
Vint traitRateOptions::_optimizeIterNum; 
Vint traitRateOptions::_optimizeStartegies;
sequenceType traitRateOptions::_sequenceType;
seqModelType traitRateOptions::_seqModelType;
MDOUBLE traitRateOptions::_initTreeLength;
string traitRateOptions::_simulationsTreeDir;
int traitRateOptions::_simulationSeqPos;
int traitRateOptions::_startSimulationsIter;
int traitRateOptions::_endSimulationsIter;
llCalc traitRateOptions::_llCalcType;
	



traitRateOptions::~traitRateOptions() 
{}

void traitRateOptions::initOptions(const string& paramFileName)
{
	initDefault();
	getParamsFromFile(paramFileName);
}

void traitRateOptions::initDefault()
{
// DEFAULTS VALUES:
	_mainType = "Optimize_Model";
    _treeFile = "";
	_treesFile = "";
	_characterFile = "";
	_seqFile = "";
	_logFile = "log.txt";
	_logValue = 5;
	_outDir = "RESULTS";
	_outFile= "traitRate.res";
	_scaledTreeFile = "scaled.tree";
	_rootFreqType = twoStateModel::STATIONARY;
	_modelType = M1;
	_rootAt = "";
	_relRate = 1.0;
	_charModelParam1 = 0.5;
	_charModelParam2 = 1.0;
	_globalRate = 1.0;
	_seqModelParam1 = 2; //that's kappa usually
	_seqModelParam2 = -1;
	_seqModelParam3 = -1;
	_seqModelParam4 = -1;
	_seqModelParam5 = -1;
	_seqModelParam6 = -1;
	_gammaCategories = 4; 
	_gammaParam = 1; 
	_treeLength = -1.0; 
	_referenceTree = "";
	_stochasicMappingIterations = 100;
	_bScaleTree = true;
	_bOptRelRate = true;
	_bOptSeqModel = true;
	_bOptCharModel = true;
	_maxOptimizationIterations = 5;
	_tolParamOptimization = 0.01; 
	_epsilonLLimprovement = 0.1; 
	_optimizePointsNum.push_back(10); 
	_optimizePointsNum.push_back(3); 
	_optimizePointsNum.push_back(1); 
	_optimizeIterNum.push_back(0); 
	_optimizeIterNum.push_back(2); 
	_optimizeIterNum.push_back(_maxOptimizationIterations); 
	_optimizeStartegies.push_back(1); 
	_optimizeStartegies.push_back(1); 
	_optimizeStartegies.push_back(0); 
	_sequenceType = NON_CODING;
	_seqModelType = JC;
	_initTreeLength = 0.0;
	_simulationsTreeDir = "";
	_simulationSeqPos = 0;
	_startSimulationsIter = 0;
	_endSimulationsIter = 0;
	_llCalcType = COMBINED;


	Parameters::addParameter("_mainType", _mainType);
	Parameters::addParameter("_treeFile", _treeFile);
	Parameters::addParameter("_treesFile", _treesFile);
	Parameters::addParameter("_characterFile", _characterFile);
	Parameters::addParameter("_seqFile", _seqFile);
	Parameters::addParameter("_logFile", _logFile);
	Parameters::addParameter("_logValue", _logValue);
	Parameters::addParameter("_outDir", _outDir);
	Parameters::addParameter("_outFile", _outFile);
	Parameters::addParameter("_scaledTreeFile", _scaledTreeFile);
	Parameters::addParameter("_rootFreqType", getRootFreqTypeStr(_rootFreqType));
	Parameters::addParameter("_modelType", getCombinedModelTypeStr(_modelType));
	Parameters::addParameter("_rootAt", _rootAt);
	Parameters::addParameter("_relRate", _relRate);
	Parameters::addParameter("_charModelParam1", _charModelParam1);
	Parameters::addParameter("_charModelParam2", _charModelParam2);
	Parameters::addParameter("_globalRate", _globalRate);
	Parameters::addParameter("_seqModelParam1", _seqModelParam1);
	Parameters::addParameter("_seqModelParam2", _seqModelParam2);
	Parameters::addParameter("_seqModelParam3", _seqModelParam3);
	Parameters::addParameter("_seqModelParam4", _seqModelParam4);
	Parameters::addParameter("_seqModelParam5", _seqModelParam5);
	Parameters::addParameter("_seqModelParam6", _seqModelParam6);
	Parameters::addParameter("_gammaCategories", _gammaCategories);
	Parameters::addParameter("_gammaParam", _gammaParam);
	Parameters::addParameter("_treeLength", _treeLength);
	Parameters::addParameter("_referenceTree", _referenceTree);
	Parameters::addParameter("_stochasicMappingIterations", _stochasicMappingIterations);
	Parameters::addParameter("_bScaleTree",(_bScaleTree == true) ? 1 : 0);
	Parameters::addParameter("_bOptRelRate",(_bOptRelRate == true) ? 1 : 0);
	Parameters::addParameter("_bOptSeqModel",(_bOptSeqModel == true) ? 1 : 0);
	Parameters::addParameter("_bOptCharModel",(_bOptCharModel == true) ? 1 : 0);
	Parameters::addParameter("_maxOptimizationIterations", _maxOptimizationIterations);
	Parameters::addParameter("_tolParamOptimization", _tolParamOptimization); 
	Parameters::addParameter("_epsilonLLimprovement", _epsilonLLimprovement); 
	Parameters::addParameter("_optimizePointsNum", getStrFromVint(_optimizePointsNum)); 
	Parameters::addParameter("_optimizeIterNum", getStrFromVint(_optimizeIterNum)); 
	Parameters::addParameter("_optimizeStartegies", getStrFromVint(_optimizeStartegies)); 
	Parameters::addParameter("_sequenceType", getSequenceTypeStr(_sequenceType));
	Parameters::addParameter("_seqModelType", getseqModelTypeStr(_seqModelType));
	Parameters::addParameter("_initTreeLength", _initTreeLength);
	Parameters::addParameter("_simulationsTreeDir", _simulationsTreeDir); 
	Parameters::addParameter("_simulationSeqPos", _simulationSeqPos); 
	Parameters::addParameter("_startSimulationsIter", _startSimulationsIter); 
	Parameters::addParameter("_endSimulationsIter", _endSimulationsIter); 
	Parameters::addParameter("_llCalcType", getLlCalcTypeStr(_llCalcType));
}

void traitRateOptions::getParamsFromFile(const string& paramFileName)
{
	ifstream params(paramFileName.c_str());
	if(params.fail())
        errorMsg::reportError("cannot open parameter file: " + paramFileName);
	if(params.good())
        Parameters::readParameters(params);
	params.close();

	_mainType = Parameters::getString("_mainType");
	_treeFile = Parameters::getString("_treeFile");
	_treesFile = Parameters::getString("_treesFile");
	_characterFile = Parameters::getString("_characterFile");
	_seqFile = Parameters::getString("_seqFile");
	_logFile= Parameters::getString("_logFile");
	_logValue = Parameters::getInt("_logValue");
	_outDir = Parameters::getString("_outDir");
	_outFile = Parameters::getString("_outFile");
	_scaledTreeFile = Parameters::getString("_scaledTreeFile");
	_rootFreqType = getRootFreqTypeFromStr(Parameters::getString("_rootFreqType"));
	_modelType = getCombinedModelTypeFromStr(Parameters::getString("_modelType"));
    _rootAt = Parameters::getString("_rootAt");
	_relRate = Parameters::getFloat("_relRate");
	_charModelParam1 = Parameters::getFloat("_charModelParam1");
	_charModelParam2 = Parameters::getFloat("_charModelParam2");
	_globalRate = Parameters::getFloat("_globalRate");
	_seqModelParam1 = Parameters::getFloat("_seqModelParam1");
	_seqModelParam2 = Parameters::getFloat("_seqModelParam2");
	_seqModelParam3 = Parameters::getFloat("_seqModelParam3");
	_seqModelParam4 = Parameters::getFloat("_seqModelParam4");
	_seqModelParam5 = Parameters::getFloat("_seqModelParam5");
	_seqModelParam6 = Parameters::getFloat("_seqModelParam6");
	_gammaCategories = Parameters::getInt("_gammaCategories");
	_gammaParam = Parameters::getFloat("_gammaParam");
	_treeLength = Parameters::getFloat("_treeLength");
	_referenceTree = Parameters::getString("_referenceTree");
	_stochasicMappingIterations = Parameters::getInt("_stochasicMappingIterations");
	_bScaleTree = (Parameters::getInt("_bScaleTree") == 1) ? true : false;
	_bOptRelRate = (Parameters::getInt("_bOptRelRate") == 1) ? true : false;
	_bOptSeqModel = (Parameters::getInt("_bOptSeqModel") == 1) ? true : false;
	_bOptCharModel = (Parameters::getInt("_bOptCharModel") == 1) ? true : false;
	_maxOptimizationIterations = Parameters::getInt("_maxOptimizationIterations");
	_tolParamOptimization = Parameters::getFloat("_tolParamOptimization"); 
	_optimizePointsNum = getVintFromStr(Parameters::getString("_optimizePointsNum"));
	_optimizeIterNum = getVintFromStr(Parameters::getString("_optimizeIterNum"));
	_optimizeStartegies = getVintFromStr(Parameters::getString("_optimizeStartegies"));
	_epsilonLLimprovement = Parameters::getFloat("_epsilonLLimprovement");
	_sequenceType = getSequenceTypeFromStr(Parameters::getString("_sequenceType"));
	_seqModelType = getseqModelTypeFromStr(Parameters::getString("_seqModelType"));
	_initTreeLength = Parameters::getFloat("_initTreeLength"); 
	_simulationsTreeDir = Parameters::getString("_simulationsTreeDir");
	_simulationSeqPos = Parameters::getInt("_simulationSeqPos");
	_startSimulationsIter = Parameters::getInt("_startSimulationsIter");
	_endSimulationsIter = Parameters::getInt("_endSimulationsIter");
	_llCalcType = getLlCalcTypeFromStr(Parameters::getString("_llCalcType"));
}


twoStateModel::rootFreqType traitRateOptions::getRootFreqTypeFromStr(const string& str)
{
	if (str == "ROOT_LL")
		return twoStateModel::ROOT_LL;
	else if (str == "STATIONARY")
		return twoStateModel::STATIONARY;
	else if (str == "EMPIRICAL")
		return twoStateModel::EMPIRICAL;
	else
        errorMsg::reportError("unknown type in traitRateOptions::getRootFreqTypeFromStr");
	return twoStateModel::ROOT_LL;
}

string traitRateOptions::getRootFreqTypeStr(twoStateModel::rootFreqType type)
{
	string res = "";
	switch (type)
	{
	case twoStateModel::ROOT_LL:
		res = "ROOT_LL";
		break;
	case twoStateModel::STATIONARY:
		res = "STATIONARY";
		break;
	case twoStateModel::EMPIRICAL:
		res = "EMPIRICAL";
		break;
	default:
		errorMsg::reportError("unknown type in traitRateOptions::getRootFreqTypeStr");
	}
	return res;
}

string traitRateOptions::getSequenceTypeStr(sequenceType type)
{
	string res = "";
	switch (type)
	{
	case CODING:
		res = "cds";
		break;
	case NON_CODING:
		res = "nc";
		break;
	case PROTEIN:
		res = "prot";
		break;
	default:
		errorMsg::reportError("unknown type in traitRateOptions::getSequenceTypeStr");
	}
	return res;
}

sequenceType traitRateOptions::getSequenceTypeFromStr(const string& str)
{
	if (str == "cds")
		return CODING;
	else if (str == "nc")
		return NON_CODING;
	else if (str == "prot")
		return PROTEIN;
	else
        errorMsg::reportError("unknown type in traitRateOptions::getSequenceTypeFromStr");
	return NON_CODING;
}

string traitRateOptions::getseqModelTypeStr(seqModelType type)
{
	string res = "";
	switch (type)
	{
	case JC:
		res = "JC";
		break;
	case K2:
		res = "K2";
		break;
	case HKY:
		res = "HKY";
		break;
	case GTR:
		res = "GTR";
		break;
	case JTT:
		res = "JTT";
		break;
	case MT_REV:
		res = "MT_REV";
		break;
	case CP_REV:
		res = "CP_REV";
		break;
	case LG:
		res = "LG";
		break;
	case WAG:
		res = "WAG";
		break;
	case AA_JC:
		res = "AA_JC";
		break;
	default:
		errorMsg::reportError("unknown type in traitRateOptions::getseqModelTypeStr");
	}
	return res;
}

seqModelType traitRateOptions::getseqModelTypeFromStr(const string& str)
{
	if (str == "JC")
		return JC;
	else if (str == "K2")
		return K2;
	else if (str == "HKY")
		return HKY;
	else if (str == "GTR")
		return GTR;
	else if (str == "JTT")
		return JTT;
	else if (str == "MT_REV")
		return MT_REV;
	else if (str == "CP_REV")
		return CP_REV;
	else if (str == "LG")
		return LG;
	else if (str == "WAG")
		return WAG;
	else if (str == "AA_JC")
		return AA_JC;
	else
        errorMsg::reportError("unknown type in traitRateOptions::getseqModelTypeFromStr");
	return JC;

}

string traitRateOptions::getCombinedModelTypeStr(combinedModelType type)
{
	string res = "";
	switch (type)
	{
	case M0:
		res = "M0";
		break;
	case M1:
		res = "M1";
		break;
	default:
		errorMsg::reportError("unknown type in traitRateOptions::getCombinedModelTypeStr");
	}
	return res;

}

combinedModelType traitRateOptions::getCombinedModelTypeFromStr(const string& str)
{
	if (str == "M0")
		return M0;
	else if (str == "M1")
		return M1;
	else
        errorMsg::reportError("unknown type in traitRateOptions::getCombinedModelTypeFromStr");
	return M1;
}

string traitRateOptions::getLlCalcTypeStr(llCalc type)
{
	string res = "";
	switch (type)
	{
	case SEQ:
		res = "SEQ";
		break;
	case COMBINED:
		res = "COMBINED";
		break;
	default:
		errorMsg::reportError("unknown type in traitRateOptions::getLlCalcTypeStr");
	}
	return res;

}

llCalc traitRateOptions::getLlCalcTypeFromStr(const string& str)
{
	if (str == "SEQ")
		return SEQ;
	else if (str == "COMBINED")
		return COMBINED;
	else
        errorMsg::reportError("unknown type in traitRateOptions::getLlCalcTypeFromStr");
	return COMBINED;
}
