#include "utils.h"
#include "treeIt.h"
#include "treeUtil.h"
#include "seqContainerTreeMap.h"

using namespace std;


//identical to the lib function checkThatNamesInTreeAreSameAsNamesInSequenceContainer() but with different error message
void checkTreeAndContainerSameTaxa(const tree& et,const sequenceContainer & sc, string inScFileName,  string inTreeFileName, bool bLeavesOnly/*=tree*/)
{
	treeIterDownTopConst tIt(et);
	for (tree::nodeP mynode = tIt.first(); mynode != tIt.end(); mynode = tIt.next()) {
		bool bFound = false;
		if (bLeavesOnly) {
            if (mynode->isInternal()) 
                continue;
		}
		sequenceContainer::constTaxaIterator it=sc.constTaxaBegin();
		for (;it != sc.constTaxaEnd(); ++it) 
		{
			string scName = it->name();
			string treeNodeName = mynode->name();

			if (it->name() == mynode->name()) 
			{
				bFound = true;
				break;
			}
		}
		if (bFound == false) 
		{
			string errMsg = "The taxon: ";
			errMsg += mynode->name();
			errMsg += " was found in the tree file: " + inTreeFileName;
			errMsg += " but not found in the data file: " + inScFileName;
			LOG(4,<<errMsg<<endl);
			errorMsg::reportError(errMsg);
		}
	}
	
	sequenceContainer::constTaxaIterator it=sc.constTaxaBegin();
	for (;it != sc.constTaxaEnd(); ++it){
		bool bFound = false;	
		for (tree::nodeP mynode = tIt.first(); mynode != tIt.end(); mynode = tIt.next()) {
			if (bLeavesOnly)
			{
                if (mynode->isInternal()) 
                    continue;
			}
			if (it->name() == mynode->name()) 
			{
				bFound = true;
				break;
			}
		}
		if (bFound == false) 
		{
			string errMsg = "The taxon: ";
			errMsg += it->name();
			errMsg += " was found in the data file: " + inScFileName + " but not found in the tree file: " + inTreeFileName;
			errorMsg::reportError(errMsg);
		}
	}
}


void printTreeStatesAsBPValues(ostream &out, Vint &states, const tree::nodeP &myNode, const alphabet* pAlph)  {
	if (myNode->isLeaf()) {
		//out << myNode->name()<< ":"<<myNode->dis2father();
		out << myNode->name()<<"-"<<pAlph->fromInt(states[myNode->id()])<< ":"<<myNode->dis2father();
		return;
	} else {
		out <<"(";
		for (int i=0;i<myNode->getNumberOfSons();++i) {
			if (i>0) out <<",";
			printTreeStatesAsBPValues(out,states,myNode->getSon(i),pAlph);
		}
		out <<")";
		//if (myNode->isRoot()==false) {
			//out<<states[myNode->id()]<<"--";
			//out<<myNode->name();
			out.precision(3);
			//out << "["<<myNode->name()<<"]"; 
			//out<<":"<<myNode->dis2father();
			out << "["<<myNode->name()<<"-"<<pAlph->fromInt(states[myNode->id()])<<"]"; 
			out<<":"<<myNode->dis2father();

		//}
	}
}

//returns the MP score of the tree assuming 2 character alphabet
int getMPScore(const tree& et,const sequenceContainer & charContainer, int pos)
{
	int nodesNum = et.getNodesNum();
	//the states can be 0, 1, or 2 where 2 represents both states are possible
	Vint states(nodesNum);
	// intialize _states veactor with values of leaves
	seqContainerTreeMap scTreeMap(charContainer, et);	
	vector <tree::nodeP> leaves;
	et.getAllLeaves(leaves, et.getRoot());
	for (int i=0; i< leaves.size();i++){
		int myleafId = leaves[i]->id();
		int mySeqId = scTreeMap.seqIdOfNodeI(myleafId);
		string name = leaves[i]->name();
		states[myleafId] = charContainer[mySeqId][pos];
	}

	int cost = 0;
	treeIterDownTopConst tIt(et);
	for (tree::nodeP mynode = tIt.first(); mynode != tIt.end(); mynode = tIt.next()) {
		if (mynode->isLeaf()) 
			continue;
		if (mynode->getNumberOfSons() > 2)
		{
			bool zeroExist = false;
			bool oneExist = false;
			bool twoExist = false;
            for (int s = 0; s < mynode->getNumberOfSons(); ++s)
			{
				int idSon = (mynode->getSon(s))->id();
				int letSon = states[idSon];
				if (letSon == 0)
					zeroExist = true;
				else if (letSon == 1)
					oneExist = true;
				else if (letSon == 2)
					twoExist = true;
			}
			if (!twoExist)
			{
				if (oneExist && zeroExist)
				{
					states[mynode->id()] = 2;
                    ++cost;
				}
				else //only one state exists
					states[mynode->id()] = states[mynode->getSon(0)->id()];
			}
			else 
			{
				if (oneExist && zeroExist)
				{
					states[mynode->id()] = 2;
                    ++cost;
				}
				else
				{
					states[mynode->id()] = states[mynode->getSon(0)->id()];
				}

			}
		}
		else
		{
			string name = mynode->name();
			int idSon0 = (mynode->getSon(0))->id();
			int idSon1 = (mynode->getSon(1))->id();
			int letSon0 = states[idSon0];
			int letSon1 = states[idSon1];
			int sum = letSon0 + letSon1;
			if (letSon0 == letSon1)
				states[mynode->id()] = letSon0;
			else if (sum == 1) {//each son has different basic state;
				states[mynode->id()] = 2;
				++cost;
			}
			else if (sum == 4) //both sons have state = 2
				states[mynode->id()] = 2;
			else { //one son is 1/0 and the other 2 - should find intersection
				int intersect = (letSon0 == 2) ? letSon1 : letSon1;
				states[mynode->id()] = intersect;
			}
		}
	}	
	cerr<<"MP cost = "<<cost<<endl;
	return cost;
}


tree::nodeP findMRCA(const tree& modelTree, const tree& newTree)
{
	int s;
	tree::nodeP outgroup = NULL;
	for (s = 0; s < modelTree.getRoot()->getNumberOfSons(); ++s)
	{
		vector<tree::nodeP> leaves;
		tree::nodeP son = modelTree.getRoot()->getSon(s);
		modelTree.getAllLeaves(leaves, son);
        outgroup = newTree.findNodeByName(leaves[0]->name());
		for (int l = 1; l < leaves.size(); ++l)
		{
			//check if the current leaf is within the current outgroup. if not then go up one level and check again
			while (!outgroup->isRoot())
			{
				string leafName = leaves[l]->name();
				if (newTree.findNodeByName(leaves[l]->name(), outgroup) == NULL)
					outgroup = outgroup->father();
				else
					break;
			}
			if (outgroup->isRoot())
				break;
		}
		if (!outgroup->isRoot())
			break;
	}
	return outgroup;
}

bool containAllNames(const tree& inTree, tree::nodeP fromHere, const vector<tree::nodeP>& leavesToLook)
{
	bool res = true;
	for (int l = 0; l < leavesToLook.size(); ++l)
	{
		if (inTree.findNodeByName(leavesToLook[l]->name(), fromHere) == NULL)
		{
			res = false;
			break;
		}
	}
	return res;
}


//given an unrooted tree - transform to to a rooted one given a model rooted tree
void makeBifurcatingTree(const string& inModelTree, const string& inUnrootedTree, const string& outTree)
{
	tree modelTree(inModelTree.c_str());
	tree unrootedTree(inUnrootedTree.c_str());
	if (modelTree.getRoot()->getNumberOfSons() != 2)
		errorMsg::reportError("model tree is assumed to be bifurcating");
	
	//1. find the lowest node in unrootedTree that contains all leaves of one of the subtrees below the root of the model tree
	vector<tree::nodeP> nodesVec;
	unrootedTree.getFromRootToLeaves(nodesVec);
	tree::nodeP outgroup = NULL;
	for (int n = 0; n < nodesVec.size(); ++n)
	{
        unrootedTree.rootAt(nodesVec[n]);
		outgroup = findMRCA(modelTree, unrootedTree);
		if (!outgroup->isRoot())
			break;
	}
	if (outgroup->isRoot())
		errorMsg::reportError("should do something: outgroup is root");

	//2. root tree at the MRCA
	tree tree1, tree2, newTree;
	if (outgroup->father() == NULL) 
		errorMsg::reportError("o ooo");
	tree::nodeP tmp = makeNodeBetweenTwoNodes(unrootedTree,outgroup->father(),outgroup, "interNode");
	unrootedTree.rootAt(tmp); // tmp is the interNode and it's now the root of the tree. Its sons are node2splitOnNewTree and its father.
	cerr<<"2. unrootedtree length = "<<getSumOfBranchLengths(unrootedTree)<<endl;
	
	//set distances to sons - have to find which subtree in the new rooted tree corresponds to son0 in modelTree
	vector<tree::nodeP> son0Decendents;
	modelTree.getAllLeaves(son0Decendents, modelTree.getRoot()->getSon(0));
	int whichSon = 1;
	if (containAllNames(unrootedTree, unrootedTree.getRoot()->getSon(0), son0Decendents))
		whichSon = 0;

	int otherSon = abs(whichSon-1);
	MDOUBLE dist0 = modelTree.getRoot()->getSon(0)->dis2father();
	MDOUBLE dist1 = modelTree.getRoot()->getSon(1)->dis2father();

	MDOUBLE totalDist = unrootedTree.getRoot()->getSon(whichSon)->dis2father() + unrootedTree.getRoot()->getSon(otherSon)->dis2father();
	MDOUBLE son0Dist = (dist0/(dist1+dist0)) * totalDist;
	MDOUBLE son1Dist = (dist1/(dist1+dist0)) * totalDist;

	unrootedTree.getRoot()->getSon(whichSon)->setDisToFather(son0Dist);
	unrootedTree.getRoot()->getSon(otherSon)->setDisToFather(son1Dist);
	unrootedTree.output(outTree, tree::PHYLIP, false);
}

MDOUBLE getSumTipLengths(const tree& inTree)
{
	vector<tree::nodeP> leaves;
	inTree.getAllLeaves(leaves, inTree.getRoot());
	MDOUBLE sum = 0.0;
	for (int l = 0; l < leaves.size(); ++l)
	{
		sum += leaves[l]->dis2father();
	}
	return sum;

}


sequenceContainer extractCodonPos(const sequenceContainer& inSc, int pos)
{
	if ((pos < 1) || (pos > 3))
		errorMsg::reportError("can only extract 1,2, or third codon position");
	sequenceContainer outSc = inSc;
	Vint posToRemove(outSc.seqLen(),1);
	
	for (int i = pos-1; i < outSc.seqLen(); i += 3) {
		posToRemove[i] = 0;
	}
	outSc.removePositions(posToRemove);
	return outSc;
}
